@@ -11,13 +11,13 @@ from Bio.KDTree import KDTree
 from luna.MyBio.selector import Selector, AtomSelector
 from luna.MyBio.util import biopython_entity_to_mol
 from luna.interaction.contact import get_proximal_compounds
+from luna.interaction.type import InteractionType
 from luna.mol.atom import ExtendedAtom, AtomData
 from luna.mol.charge_model import OpenEyeModel
-from luna.wrappers.base import MolWrapper
 from luna.mol.features import ChemicalFeature
+from luna.wrappers.base import MolWrapper
 from luna.util.exceptions import MoleculeSizeError, IllegalArgumentError
 from luna.util.default_values import ACCEPTED_MOL_OBJ_TYPES, COV_SEARCH_RADIUS
-from luna.interaction.type import InteractionType
 from luna.util import math as im
 from luna.util.file import pickle_data, unpickle_data
 from luna.version import __version__
@@ -58,6 +58,8 @@ class AtomGroupsManager():
 
         self.version = __version__
 
+        self._compounds = set()
+
         self.add_atm_grps(atm_grps)
 
     @property
@@ -66,6 +68,12 @@ class AtomGroupsManager():
         Additional objects should be added using the method :py:meth:`add_atm_grps`."""
         return self._atm_grps
 
+    @property
+    def compounds(self):
+        """set of :class:`~luna.MyBio.PDB.Residue.Residue`, read-only:
+        Compounds comprising the ``atm_grps``."""
+        return self._compounds
+
     @property
     def child_dict(self):
         """dict, read-only: Mapping between atoms (`ExtendedAtom`) and atom groups (`AtomGroup`).
@@ -159,6 +167,7 @@ class AtomGroupsManager():
 
         for atm_grp in atm_grps:
             self.child_dict[tuple(sorted(atm_grp.atoms))] = atm_grp
+            self._compounds.update(atm_grp.compounds)
             atm_grp.manager = self
 
     def remove_atm_grps(self, atm_grps):
@@ -235,7 +244,7 @@ class AtomGroupsManager():
         # Hydrophobic islands dictionary. Keys are integer values and items are defined by a set of atom groups.
         hydrop_islands = defaultdict(set)
 
-        # It stores a mapping of an atom (represented by its serial number) and a hydrophobic island (defined by its keys).
+        # It stores a mapping of an atom (represented by its full id) and a hydrophobic island (defined by its keys).
         atm_mapping = {}
 
         island_id = 0
@@ -244,7 +253,7 @@ class AtomGroupsManager():
             atm = atm_grp.atoms[0]
 
             # Recover the groups of all neighbors of this atom (it will merge all existing islands).
-            nb_grps = set([atm_mapping[nbi.serial_number] for nbi in atm.neighbors_info if nbi.serial_number in atm_mapping])
+            nb_grps = set([atm_mapping[nbi.full_id] for nbi in atm.neighbors_info if nbi.full_id in atm_mapping])
 
             # Already there are hydrophobic islands formed by the neighbors of this atom.
             if nb_grps:
@@ -258,10 +267,11 @@ class AtomGroupsManager():
                         atm_mapping[k] = island_id
 
                 hydrop_islands[island_id] = new_island
-                atm_mapping[atm.serial_number] = island_id
+                atm_mapping[atm.get_full_id()] = island_id
             else:
-                atm_mapping[atm.serial_number] = island_id
+                atm_mapping[atm.get_full_id()] = island_id
                 hydrop_islands[island_id].add(atm)
+
             island_id += 1
 
         # Create AtomGroup objects for the hydrophobic islands
@@ -278,7 +288,7 @@ class AtomGroupsManager():
             trgt_atm = inter.trgt_grp.atoms[0]
 
             # The two island ids are used as key.
-            key = tuple(sorted([atm_mapping[src_atm.serial_number], atm_mapping[trgt_atm.serial_number]]))
+            key = tuple(sorted([atm_mapping[src_atm.get_full_id()], atm_mapping[trgt_atm.get_full_id()]]))
 
             island_island_inter[key].add(inter)
 
@@ -289,8 +299,8 @@ class AtomGroupsManager():
                 src_atm = inter.src_grp.atoms[0]
                 trgt_atm = inter.trgt_grp.atoms[0]
 
-                island_atms[atm_mapping[src_atm.serial_number]].add(src_atm)
-                island_atms[atm_mapping[trgt_atm.serial_number]].add(trgt_atm)
+                island_atms[atm_mapping[src_atm.get_full_id()]].add(src_atm)
+                island_atms[atm_mapping[trgt_atm.get_full_id()]].add(trgt_atm)
 
             centroid1 = im.centroid(im.atom_coordinates(island_atms[k[0]]))
             centroid2 = im.centroid(im.atom_coordinates(island_atms[k[1]]))
@@ -761,15 +771,12 @@ class AtomGroupPerceiver():
         If not None, add hydrogens appropriate for pH ``ph``.
     amend_mol : bool
         If True, apply validation and standardization of molecules read from a PDB file.
+    charge_model : class:`~luna.mol.charge_model.ChargeModel`
+        A charge model object. By default, the implementation of OpenEye charge model is used.
     mol_obj_type : {"rdkit", "openbabel"}
         If "rdkit", parse the converted molecule with RDKit and return an instance of :class:`rdkit.Chem.rdchem.Mol`.
         If "openbabel", parse the converted molecule with Open Babel and return an instance of
         :class:`openbabel.pybel.Molecule`.
-    charge_model : class:`~luna.mol.charge_model.ChargeModel`
-        A charge model object. By default, the implementation of OpenEye charge model is used.
-    tmp_path : str, optional
-        A temporary directory to where temporary files will be saved.
-        If not provided, the system's default temporary directory will be used instead.
     expand_selection : bool
         If True (the default), perceive features for a given molecule considering all nearby molecules.
         The goal is to identify any covalently bonded molecules that may alter the
@@ -784,6 +791,9 @@ class AtomGroupPerceiver():
     radius : float
         If ``expand_selection`` is True, select all molecules up to a maximum of ``radius`` away (measured in Å).
         The default value is 2.2, which comprises covalent bond distances.
+    tmp_path : str, optional
+        A temporary directory to where temporary files will be saved.
+        If not provided, the system's default temporary directory will be used instead.
     critical : bool
         If False, ignore any errors during the processing a molecule and continue to the next one.
         The default value is True, which implies that any errors will raise an exception.
@@ -794,30 +804,37 @@ class AtomGroupPerceiver():
         If ``mol_obj_type`` is not either 'rdkit' nor 'openbabel'.
     """
 
-    def __init__(self, feature_extractor, add_h=False, ph=None, amend_mol=True, mol_obj_type="rdkit",
-                 charge_model=OpenEyeModel(), tmp_path=None, expand_selection=True, radius=COV_SEARCH_RADIUS,
-                 critical=True):
+    def __init__(self, feature_extractor, add_h=False, ph=None, amend_mol=True,
+                 charge_model=OpenEyeModel(), mol_obj_type="rdkit",
+                 expand_selection=True, radius=COV_SEARCH_RADIUS,
+                 cache=None, tmp_path=None, critical=True):
 
         if mol_obj_type not in ACCEPTED_MOL_OBJ_TYPES:
-            raise IllegalArgumentError("Objects of type '%s' are not currently accepted. "
-                                       "The available options are: %s." % (mol_obj_type, ", ".join(ACCEPTED_MOL_OBJ_TYPES)))
+            acc_mol_obj_types = ", ".join(ACCEPTED_MOL_OBJ_TYPES)
+            raise IllegalArgumentError("Objects of type '%s' are not currently"
+                                       "accepted. The available options are: "
+                                       "%s." % (mol_obj_type,
+                                                acc_mol_obj_types))
 
         self.feature_extractor = feature_extractor
 
         self.add_h = add_h
         self.ph = ph
-        # If the user decided not to add hydrogens it will try to use the existing ones.
+        # If the user decided not to add hydrogens,
+        # it will try to use the existing ones.
         self.keep_hydrog = not self.add_h
 
         self.amend_mol = amend_mol
-        self.mol_obj_type = mol_obj_type
         self.charge_model = charge_model
-        self.tmp_path = tmp_path
+        self.mol_obj_type = mol_obj_type
         self.expand_selection = expand_selection
         self.radius = radius
 
-        # If the pharmacophoric perception is critical, any exception during the processing of
-        # a molecule will stop the whole processing.
+        self.cache = cache
+        self.tmp_path = tmp_path
+
+        # If the pharmacophoric perception is critical, any exception during
+        # the processing of a molecule will stop the whole processing.
         self.critical = critical
 
     def perceive_atom_groups(self, compounds, mol_objs_dict=None):
@@ -843,11 +860,18 @@ class AtomGroupPerceiver():
         mol_objs_dict = mol_objs_dict or {}
 
         self.atm_grps_mngr = AtomGroupsManager()
-
         self.atm_mapping = {}
 
         compounds = set(compounds)
-        # Controls which compounds are allowed to expand when this option is set ON.
+        init_comps_set = set(compounds)
+
+        if self.cache:
+            cached_compounds = set([c for c in compounds
+                                    if self.cache.is_compound_cached(c)])
+            compounds = compounds - cached_compounds
+
+        # Controls which compounds are allowed to expand
+        # when 'expand_selection' is set ON.
         pruned_comps = set()
         # Create a queue of compounds to be processed.
         comp_queue = set(compounds)
@@ -856,46 +880,125 @@ class AtomGroupPerceiver():
             try:
                 comp = comp_queue.pop()
 
-                logger.debug("It will try to perceive the features of the compound %s as no predefined properties "
-                             "was provided." % comp)
+                logger.debug("It will try to perceive the features of the "
+                             "compound '%s' as no predefined properties "
+                             "were provided." % comp.full_name)
 
                 mol_obj = mol_objs_dict.get(comp.id, None)
 
-                # If no OBMol object was defined and expand_selection was set ON and it is not a border compound,
-                # it will get all compounds around the target and create a new OBMol object with them.
+                # If no OBMol object was defined and expand_selection
+                # was set ON and it is not a border compound, it will
+                # get all compounds around the target and create a new
+                # OBMol object with them.
                 if mol_obj is None and self.expand_selection:
-                    comp_list = get_proximal_compounds(comp)
+                    # Remove the ligand from the list when
+                    # it was provided as an OBMol object.
+                    comp_list = [c for c in get_proximal_compounds(comp)
+                                 if c.id not in mol_objs_dict]
 
-                    # Expands the queue of compounds with any new border compound.
+                    # Expands the queue of compounds
+                    # with any new border compound.
                     if comp not in pruned_comps:
                         border_comps = set(comp_list) - compounds
                         pruned_comps |= border_comps
                         comp_queue |= border_comps
 
-                # Otherwise, the compound list will be composed only by the target compound.
+                # Otherwise, the compound list will be composed
+                # only by the target compound.
                 else:
                     comp_list = [comp]
 
                 # Recover all atoms from the previous selected compounds.
-                selector = Selector(keep_altloc=False, keep_hydrog=self.keep_hydrog)
-                atoms = tuple([a for r in comp_list for a in r.get_unpacked_list() if selector.accept_atom(a)])
+                selector = Selector(keep_altloc=False,
+                                    keep_hydrog=self.keep_hydrog)
+                atoms = tuple([a for r in comp_list
+                               for a in r.get_unpacked_list()
+                               if selector.accept_atom(a)])
 
                 self._assign_properties(comp, atoms, mol_obj)
             except Exception:
-                logger.debug("Features for the compound '%s' were not correctly perceived." % comp)
+                logger.debug("Features for the compound '%s' were "
+                             "not correctly perceived." % comp)
 
                 if self.critical:
                     raise
 
-        # Remove atom groups not comprising the provided compound list (parameter 'compounds').
+        if self.cache and cached_compounds:
+            self._apply_cache(cached_compounds, comp.get_parent_by_level("M"))
+
+        # Remove atom groups not comprising the provided
+        # compound list (parameter 'compounds').
         remove_atm_grps = []
         for atm_grp in self.atm_grps_mngr:
-            if any([c in compounds for c in atm_grp.compounds]) is False:
+            if any([c in init_comps_set for c in atm_grp.compounds]) is False:
                 remove_atm_grps.append(atm_grp)
         self.atm_grps_mngr.remove_atm_grps(remove_atm_grps)
 
         return self.atm_grps_mngr
 
+    def _apply_cache(self, compounds, model):
+
+        def copy_atom(ref_atm, force_update=False):
+            ref_res = ref_atm.parent
+            ref_chain = ref_res.parent
+
+            # Recover current structure entities.
+            chain = model[ref_chain.id]
+            res = chain[ref_res.id]
+            atm = res[ref_atm.id]
+
+            # If an ExtendedAtom already exists, return it.
+            if not force_update and atm in self.atm_mapping:
+                return self.atm_mapping[atm]
+
+            # Define a new ExtendedAtom.
+            ext_atm = self._new_extended_atom(atm)
+
+            # Set its atomic invariants.
+            ext_atm.invariants = ref_atm.invariants
+
+            # Add neighbors' information to the new ExtendedAtom.
+            for nbi in ref_atm.neighbors_info:
+                atom_info = AtomData(nbi.atomic_num,
+                                     nbi.coord, nbi.bond_type,
+                                     nbi.full_id)
+                ext_atm.add_nb_info([atom_info])
+
+            return ext_atm
+
+        comp_names = set([c.full_name for c in compounds])
+
+        for atm_grp in sorted(self.cache.atm_grps_mngr):
+            if any([c.full_name in comp_names for c in atm_grp.compounds]):
+                atoms = []
+                for atm in atm_grp.atoms:
+                    # Copy extended atom, i.e., create a new extended
+                    # atom or recover an existing one and set invariants
+                    # and neighbors' information when necessary.
+                    ext_atm = copy_atom(atm, force_update=True)
+                    atoms.append(ext_atm)
+
+                # Create the new atom group.
+                features = atm_grp.features
+                self.atm_grps_mngr.new_atm_grp(atoms, features)
+
+        for edge in sorted(self.cache.atm_grps_mngr.graph.edges):
+            # Skip already existing edges.
+            if edge in self.atm_grps_mngr.graph.edges:
+                continue
+
+            atoms = []
+            for atm in edge:
+                # Copy extended atom, i.e., create a new
+                # extended atom or recover an existing one
+                # and set invariants and neighbors' information
+                # if necessary.
+                ext_atm = copy_atom(atm)
+                atoms.append(ext_atm)
+
+            # Update the AtomGroupsManager object with a new edge.
+            self.atm_grps_mngr.graph.add_edge(atoms[0], atoms[1], weight=1)
+
     def _assign_properties(self, target_compound, target_atoms, mol_obj=None):
 
         # If no OBMol was defined, create a new one with the compound list.
@@ -919,14 +1022,14 @@ class AtomGroupPerceiver():
         atm_map = {}
         trgt_atms = {}
         for i, atm_obj in enumerate(atm_obj_list):
-            atm_map[atm_obj.get_idx()] = target_atoms[i].serial_number
+            atm_map[atm_obj.get_idx()] = target_atoms[i].get_full_id()
 
-            trgt_atms[target_atoms[i].serial_number] = self._new_extended_atom(target_atoms[i])
+            trgt_atms[target_atoms[i].get_full_id()] = self._new_extended_atom(target_atoms[i])
 
             # Invariants are still None, so it means a new ExtendedAtom has just been created.
-            if trgt_atms[target_atoms[i].serial_number].invariants is None:
+            if trgt_atms[target_atoms[i].get_full_id()].invariants is None:
                 # Update atomic invariants for new ExtendedAtoms created.
-                trgt_atms[target_atoms[i].serial_number].invariants = atm_obj.get_atomic_invariants()
+                trgt_atms[target_atoms[i].get_full_id()].invariants = atm_obj.get_atomic_invariants()
 
             # The current atom already has its invariants updated, which means the atom was created previously
             # by another target compound.
@@ -942,9 +1045,9 @@ class AtomGroupPerceiver():
                 # However, we need to highlight that the main goal of selecting atoms using COV_SEARCH_RADIUS is to find atoms
                 # covalently bonded to the current compound. Therefore, atoms in the border of nearby molecules are not important
                 # right now and they will be updated in their own time.
-                if trgt_atms[target_atoms[i].serial_number].parent == target_compound:
+                if trgt_atms[target_atoms[i].get_full_id()].parent == target_compound:
                     # Update the atomic invariants for an ExtendedAtom.
-                    trgt_atms[target_atoms[i].serial_number].invariants = atm_obj.get_atomic_invariants()
+                    trgt_atms[target_atoms[i].get_full_id()].invariants = atm_obj.get_atomic_invariants()
 
         # Set all neighbors, i.e., covalently bonded atoms.
         for bond_obj in mol_obj.get_bonds():
@@ -958,9 +1061,9 @@ class AtomGroupPerceiver():
                     # If the current bgn_atm_obj consists of an atom from the current target compound, we can update its information.
                     # Other compounds are ignored for now as they will have their own time to update its information.
                     if trgt_atms[atm_map[bgn_atm_obj.get_idx()]].parent == target_compound:
-                        serial_number = atm_map.get(end_atm_obj.get_idx())
+                        full_id = atm_map.get(end_atm_obj.get_idx())
                         coord = mol_obj.get_atom_coord_by_id(end_atm_obj.get_id())
-                        atom_info = AtomData(end_atm_obj.get_atomic_num(), coord, bond_obj.get_bond_type(), serial_number)
+                        atom_info = AtomData(end_atm_obj.get_atomic_num(), coord, bond_obj.get_bond_type(), full_id)
                         trgt_atms[atm_map[bgn_atm_obj.get_idx()]].add_nb_info([atom_info])
 
                 # If the atom 2 is not a hydrogen, add atom 1 to its neighbor list.
@@ -968,9 +1071,9 @@ class AtomGroupPerceiver():
                     # If the current end_atm_obj consists of an atom from the current target compound, we can update its information.
                     # Other compounds are ignored for now as they will have their own time to update its information.
                     if trgt_atms[atm_map[end_atm_obj.get_idx()]].parent == target_compound:
-                        serial_number = atm_map.get(bgn_atm_obj.get_idx())
+                        full_id = atm_map.get(bgn_atm_obj.get_idx())
                         coord = mol_obj.get_atom_coord_by_id(bgn_atm_obj.get_id())
-                        atom_info = AtomData(bgn_atm_obj.get_atomic_num(), coord, bond_obj.get_bond_type(), serial_number)
+                        atom_info = AtomData(bgn_atm_obj.get_atomic_num(), coord, bond_obj.get_bond_type(), full_id)
                         trgt_atms[atm_map[end_atm_obj.get_idx()]].add_nb_info([atom_info])
 
         group_features = self.feature_extractor.get_features_by_groups(mol_obj, atm_map)
@@ -986,13 +1089,12 @@ class AtomGroupPerceiver():
 
         # Update the graph in the AtomGroupsManager object with the current compound information.
         for mybio_atm in target_compound.get_atoms():
-            if mybio_atm.serial_number in trgt_atms and mybio_atm.parent == target_compound:
-                atm = trgt_atms[mybio_atm.serial_number]
+            if mybio_atm.get_full_id() in trgt_atms and mybio_atm.parent == target_compound:
+                atm = trgt_atms[mybio_atm.get_full_id()]
 
                 for nb_info in atm.neighbors_info:
-                    if nb_info.serial_number in trgt_atms:
-                        self.atm_grps_mngr.graph.add_edge(atm, trgt_atms[nb_info.serial_number], weight=1)
-
+                    if nb_info.full_id in trgt_atms:
+                        self.atm_grps_mngr.graph.add_edge(atm, trgt_atms[nb_info.full_id], weight=1)
         return True
 
     def _new_extended_atom(self, atm, invariants=None):
